@@ -3,9 +3,11 @@ import os
 import time
 
 import features
+import threading
 
 from app import app
 from data.secscan_model import secscan_model
+from workers.gunicorn_worker import GunicornWorker
 from workers.worker import Worker
 from util.log import logfile_path
 from endpoints.v2 import v2_bp
@@ -30,6 +32,20 @@ class SecurityWorker(Worker):
         self._next_token = self._model.perform_indexing(self._next_token)
 
 
+def create_gunicorn_worker():
+    """
+    follows the gunicorn application factory pattern, enabling
+    a quay worker to run as a gunicorn worker thread.
+
+    this is useful when utilizing gunicorn's hot reload in local dev.
+
+    utilizing this method will enforce a 1:1 quay worker to gunicorn worker ratio.
+    """
+    app.register_blueprint(v2_bp, url_prefix="/v2")
+    worker = GunicornWorker(__name__, app, SecurityWorker(), features.SECURITY_SCANNER)
+    return worker
+
+
 if __name__ == "__main__":
     if os.getenv("PYDEV_DEBUG", None):
         import pydevd_pycharm
